import org.apache.axis2.client.*;
import org.apache.axis2.*;
import org.apache.axis2.context.*;
import org.apache.axiom.om.*;
import org.apache.axiom.soap.*;
import javax.xml.namespace.*;

import SoaG.*;
import SoaG.AdabasEmployeesMiniServiceStub.*;

/*
 *   This tutorial selects Employees records by multiple ranges of Personnel IDs
 *   and asks the SOA Gateway to return a "chunk" of 20 records per call.
 *   
 *   This requires a Conversation to be initiated, this is done with a SELECT
 *   call as usual, to retrieve the following chunk(s) the selectNext method
 *   has to be used, the Conversation is terminated with a selectEnd call.
 */
public class Tut_03c_SelectConversational {

	public static void main(String[] args) {

	    try {
		  AdabasEmployeesMiniServiceStub stub = new AdabasEmployeesMiniServiceStub();

		  AdabasEmployeeSelectType selType = new AdabasEmployeesMiniServiceStub.AdabasEmployeeSelectType();
		  
		  /*
		   *   The first condition block selects personnel_id between 11100111 and 11100222
		   *   Conditions within one block are ANDed, so the following means
		   *     personnel_id >= 11100111 AND personnel_id <= 11100222
		   */
		  Condition_type0 cond1 = new Condition_type0();
		  
		  Personnel_id_type0 p1From = new Personnel_id_type0();
		  p1From.setCondition(ConditionType.GE);
		  p1From.setString("11100111");
		  cond1.addPersonnel_id(p1From);
		  
		  Personnel_id_type0 p1To = new Personnel_id_type0();
		  p1To.setCondition(ConditionType.LE);
		  p1To.setString("11100222");
		  cond1.addPersonnel_id(p1To);
		  		  
		  /*
		   *   The second condition block selects personnel_id between 50005000 and 500079999
		   */
		  Condition_type0 cond2 = new Condition_type0();
		  
		  Personnel_id_type0 p2From = new Personnel_id_type0();
		  p2From.setCondition(ConditionType.GE);
		  p2From.setString("50005000");
		  cond2.addPersonnel_id(p2From);
		  
		  Personnel_id_type0 p2To = new Personnel_id_type0();
		  p2To.setCondition(ConditionType.LE);
		  p2To.setString("50007999");
		  cond2.addPersonnel_id(p2To);
		  
		  /* 
		   *     Multiple condition blocks are ORed, so we then have
		   *     personnel_id >= 11100111 AND personnel_id <= 11100222 OR
		   *     personnel_id >= 50005000 AND personnel_id <= 50007999
		   */
		  selType.addCondition(cond1);
		  selType.addCondition(cond2);

		  AdabasEmployeeSelectElement ese = new AdabasEmployeesMiniServiceStub.AdabasEmployeeSelectElement();
		  ese.setAdabasEmployeeSelectElement(selType);

		  /*
		   *   Now we need to set a number of options in our outbound SOAP Header:
		   */
		  AdabasEmployeeHeader hdr = new AdabasEmployeesMiniServiceStub.AdabasEmployeeHeader();
		  hdr.setVersion(1);	// This is required
		  hdr.setConversationId("");
		  hdr.setConversationState("New");
		  hdr.setSOAGateway_Internal_Engine_MaximumRows(20);
		  
		  AdabasEmployeesMiniElement selResult = null;
		  
		  /*
		   *   SELECT he first chunk of 20 records
		   */
		  selResult = stub.select(ese, null, hdr);
		  
		  /*
		   *   In order to continue the conversation we need to extract the Conversation ID
		   *   from the inbound SOAP Headers.
		   *   
		   *   Axis2 tries hard to make life as hard as possible when trying to get at
		   *   the inbound headers, but we'll find them anyway:
		   */
		  ServiceClient sc = stub._getServiceClient();
		  OperationContext oc = sc.getLastOperationContext();
		  MessageContext inMc = oc.getMessageContext("In");
		  OMElement envelope = inMc.getEnvelope();
		  
		  /*
		   *  Now that we've run down the chain we should finally have arrived at
		   *  the SOAPHeader
		   */
		  OMElement first = envelope.getFirstElement();
		  if (first == null) {
			  System.out.println("We did NOT receive a header as expected - quitting");
			  System.exit(-99);
		  }
		  if (first.getLocalName().compareTo("Header") != 0) {
			  System.out.println("We did NOT receive a header as expected - quitting");
			  System.exit(-99);
		  }
		  OMElement adaHdr = first.getFirstElement();
		  if (adaHdr.getLocalName().compareTo("adabasEmployeeHeader") != 0) {
			  System.out.println("We did NOT receive an Adabas Employees header as expected - quitting");
			  System.exit(-99);
		  }
		  java.util.Iterator<OMElement> itHeader = adaHdr.getChildElements();
		  
		  /*
		   *   Now extract and print the individual header elements we received,
		   *   extract the ConversationId and retain it for future reference
		   */
		  String convId = "";
		  while (itHeader.hasNext()) {
			  OMElement elHdr = itHeader.next();
			  String elName = elHdr.getLocalName();
			  String elText = elHdr.getText();
			  System.out.println("Hdr = '" + elName + "', value = '" + elText + "'");
			  if (elName.compareTo("ConversationId") == 0) {
				  convId = elText;
			  }
		  }

		  int iChunk = 1;
		  printData(selResult, iChunk++);

		  /*
		   *   SELECT initiated a conversation as we've asked for it by specifying 
		   *   ConversationState=New 
		   *   For all further SelectNext requests we reference this conversation
		   *   by specifying ConversationState=Old and ConversationId=<convid_from_header> 
		   */
		  hdr.setConversationState("Old");
		  hdr.setConversationId(convId);

		  /*
		   *   Now read the remaining records in a loop,
		   *   20 records per chunk (as specified in the header)
		   */
		  AdabasEmployeeSelectNextElement sne = new AdabasEmployeesMiniServiceStub.AdabasEmployeeSelectNextElement();
		  AdabasEmployeeSelectNextType    snt = new AdabasEmployeesMiniServiceStub.AdabasEmployeeSelectNextType();
		  sne.setAdabasEmployeeSelectNextElement(snt);
		  while (1 > 0) {
			try {
			  selResult = stub.selectNext(sne, null, hdr);
			  printData(selResult, iChunk++);
			} catch (Exception ex) {
              if (ex instanceof AxisFault) {
            	AxisFault af = (AxisFault)ex;
            	SOAPFaultDetail sfd = af.getFaultDetailElement();
            	OMElement sfCode = sfd.getFirstChildWithName(new QName(null, "code"));
            	if (sfCode.getText().compareTo("-33045") == 0) {
            	  /*
            	   *   A fault with code = -33045 denotes "end of data"
            	   */
            	  System.out.println("\nEnd of data");
            	}
              } else {
			    ex.printStackTrace();
              }
			  break;
			}
		  }

          /*
		   *   We are through, end the conversation
		   */
		  AdabasEmployeeSelectEndElement see = new AdabasEmployeesMiniServiceStub.AdabasEmployeeSelectEndElement();
		  AdabasEmployeeSelectEndType    set = new AdabasEmployeesMiniServiceStub.AdabasEmployeeSelectEndType();
		  see.setAdabasEmployeeSelectEndElement(set);
		  hdr.setConversationState("End");
		  
          AcknowledgementElement ack = stub.selectEnd(see, null, hdr);
          
          System.out.println("\n\n" + ack.getAcknowledgement());
	    
	    } catch (Exception ex) {
		  ex.printStackTrace();
	    }
	    
	  }

  private static void printData( AdabasEmployeesMiniElement selResult, int iChunk ) {
		
    AdabasEmployeesMiniElementType selResType = selResult.getAdabasEmployeesMiniElement();
    AdabasEmployeesMiniType selResMiniType = selResType.getAdabasEmployeesMini();
    AdabasEmployeeType empls[] = selResMiniType.getAdabasEmployee();

    for (int i = 0; i < empls.length; i++) {
      AdabasEmployeeType r = empls[i];
      System.out.println("[" + iChunk + "," + i + "] = "
					         + r.getPersonnel_id() + ", "
					         + r.getName() + ", "
					         + r.getFirst_name() + ", "
					         + r.getCity());
    }
  }
}

