import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAPFaultDetail;
import org.apache.axis2.AxisFault;
import org.apache.axis2.client.ServiceClient;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.context.OperationContext;

import SoaG.*;
import SoaG.AdabasEmployeesMiniServiceStub.*;

/*
 *   We are first going to ADD a record in the context of a transaction and
 *   back it out again. To prove the transaction works as expected we will
 *   issue a GET within the transaction and again after the backout.
 *   
 */

public class Tut_04b_AddTransactional {

  public static void main(String[] args) {

	AdabasEmployeesMiniElement result   = null;
	AdabasEmployeesMiniServiceStub stub = null;
	AdabasEmployeePrimaryKeyType     pk = null;
    OperationResult               opRes = null;
    AdabasEmployeeHeader            hdr = null; 
    String txId    =  "";
    String convId  =  "";
	  

    try {
      stub = new AdabasEmployeesMiniServiceStub();
      
	  /*
	   *   First add a record
	   */
	  AdabasEmployeesMiniAddElement addEle = new AdabasEmployeesMiniAddElement();
	  AdabasEmployeesMiniType      theType = new AdabasEmployeesMiniType();
	  AdabasEmployeeType        theDatas[] = new AdabasEmployeeType[1];
	  AdabasEmployeeType           theData = new AdabasEmployeeType();
	  
	  theData.setPersonnel_id("99998888");
	  theData.setName("Tester");
	  theData.setFirst_name("Thename");
	  theData.setCity("Anytown");
	  String address[] = {"Mystreet", "PO box 222"};
	  theData.setAddress_line(address);
	  
	  theDatas[0] = theData;
	  theType.setAdabasEmployee(theDatas);
	  addEle.setAdabasEmployeesMiniAddElement(theType);

	  /*
	   *   Now we need to set a number of options in our outbound SOAP Header
	   *   to initiate the transaction.
	   *   
	   *   A transaction must be part of a conversation, so we need to start
	   *   a conversation at the same time.
	   */
	  hdr = new AdabasEmployeesMiniServiceStub.AdabasEmployeeHeader();
	  hdr.setVersion(1);	// This is required
	  hdr.setConversationId("");
	  hdr.setConversationState("New");
	  hdr.setTransactionId("");
	  hdr.setTransactionState("New");

	  opRes = stub.add(addEle, null, hdr);
	  
	  String sRes = opRes.getResults();
	  if (sRes.compareTo("add successful") == 0) {
	    System.out.println(sRes);
	  }

	  /*
	   *   In order to continue the conversation we need to extract the Conversation ID
	   *   from the inbound SOAP Headers.
	   *   
	   *   Axis2 tries hard to make life as hard as possible when trying to get at
	   *   the inbound headers, but we'll find them anyway:
	   */
	  ServiceClient sc = stub._getServiceClient();
	  OperationContext oc = sc.getLastOperationContext();
	  MessageContext inMc = oc.getMessageContext("In");
	  OMElement envelope = inMc.getEnvelope();
	  
	  /*
	   *  Now that we've run down the chain we should finally have arrived at
	   *  the SOAPHeader
	   */
	  OMElement first = envelope.getFirstElement();
	  if (first == null) {
		  System.out.println("We did NOT receive a header as expected - quitting");
		  System.exit(-99);
	  }
	  if (first.getLocalName().compareTo("Header") != 0) {
		  System.out.println("We did NOT receive a header as expected - quitting");
		  System.exit(-99);
	  }
	  OMElement adaHdr = first.getFirstElement();
	  if (adaHdr.getLocalName().compareTo("adabasEmployeeHeader") != 0) {
		  System.out.println("We did NOT receive an Adabas Employees header as expected - quitting");
		  System.exit(-99);
	  }
	  java.util.Iterator<OMElement> itHeader = adaHdr.getChildElements();
	  
	  /*
	   *   Now extract and print the individual header elements we received,
	   *   extract the ConversationId and retain it for future reference
	   */
	  while (itHeader.hasNext()) {
		  OMElement elHdr = itHeader.next();
		  String elName = elHdr.getLocalName();
		  String elText = elHdr.getText();
		  System.out.println("Hdr = '" + elName + "', value = '" + elText + "'");
		  if (elName.compareTo("TransactionId") == 0) {
			  txId = elText;
		  }
		  if (elName.compareTo("ConversationId") == 0) {
			  convId = elText;
		  }
	  }

    } catch (Exception ex) {
      System.out.println("--- ADD failed ---");
	  ex.printStackTrace();
	  System.exit(99);
    }

    /*
     *   Issue the following GET in the context of the open transaction.
     *   
     *   No END TRANSACTION (Commit) has been issued so this results in a so
     *   called "dirty read", which means the record has not actually been
     *   stored on the database yet, but is visible to the outside world.
     *   
     *   A backout will be issued after the GET has finished
     */
    hdr.setConversationState("End");
    hdr.setConversationId(convId);
    hdr.setTransactionState("Backout");
    hdr.setTransactionId(txId);

    /*
     *   Now GET the record just stored.
     */
    try {
  	  pk = new AdabasEmployeePrimaryKeyType();
  	  pk.setPersonnel_id("99998888");
  	  	  
  	  AdabasEmployeeGetElement getEle = new AdabasEmployeeGetElement();
  	  getEle.setAdabasEmployeeGetElement(pk);
  	  
  	  result = stub.get(getEle, null, hdr);
  	  
  	  AdabasEmployeesMiniElementType root = result.getAdabasEmployeesMiniElement();
  	  AdabasEmployeesMiniType group = root.getAdabasEmployeesMini();
  	  AdabasEmployeeType elements[] = group.getAdabasEmployee();
  	  
  	  AdabasEmployeeType r = elements[0];
      System.out.println("\n--- record read ---\n"
        			   + r.getPersonnel_id() + ", "
  		               + r.getName() + ", "
  			           + r.getFirst_name() + ", "
  			           + r.getCity()
  			           + "\n");
  	  
    } catch (Exception ex) {
      System.out.println("--- GET (conversational) failed ---");
  	  ex.printStackTrace();
  	  System.exit(99);
    }

      
    /*
     *   Issue the following GET in the context of the open transaction.
     *   
     *   No END TRANSACTION (Commit) has been issued so this results in a so
     *   called "dirty read", which means the record has not actually been
     *   stored on the database yet, but is visible to the outside world.
     *   
     *   A backout will be issued after the GET has finished
     */
    hdr.setTransactionState("Backout");
    hdr.setTransactionId(txId);

    /*
     *   Now try to GET the record again
     *   
     *   This should fail as we have issued a backout.
     */
    try {
      AdabasEmployeeGetElement getEle = new AdabasEmployeeGetElement();
      getEle.setAdabasEmployeeGetElement(pk);

      result = stub.get(getEle, null, null);

      AdabasEmployeesMiniElementType root = result.getAdabasEmployeesMiniElement();
      AdabasEmployeesMiniType group = root.getAdabasEmployeesMini();
      AdabasEmployeeType elements[] = group.getAdabasEmployee();

      AdabasEmployeeType r = elements[0];
      System.out.println("\n--- record read ---\n"
          			   + r.getPersonnel_id() + ", "
          			   + r.getName() + ", "
          			   + r.getFirst_name() + ", "
          			   + r.getCity()
          			   + "\n");
    	  
    } catch (Exception ex) {
      System.out.println("--- GET (non-conversational) failed ---");
      if (ex instanceof AxisFault) {
      	AxisFault af = (AxisFault)ex;
      	SOAPFaultDetail sfd = af.getFaultDetailElement();
      	OMElement sfCode = sfd.getFirstChildWithName(new QName(null, "code"));
      	if (sfCode.getText().compareTo("-31009") == 0) {
      	  /*
      	   *   A fault with code = -31009 denotes "record does not exist"
      	   */
      	  System.out.println("The record does not exist -> it has been backed out as expected");
      	}
      } else {
        ex.printStackTrace();
      }

    }
  }
}
