package com.risaris.axis2.entirex.transports;

import java.io.ByteArrayOutputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLDecoder;
import java.util.HashMap;

import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.TransportOutDescription;
import org.apache.axis2.handlers.AbstractHandler;
import org.apache.axis2.transport.TransportSender;
import org.apache.axis2.transport.TransportUtils;

import com.softwareag.entirex.aci.Broker;
import com.softwareag.entirex.aci.BrokerService;
import com.softwareag.entirex.aci.BrokerMessage;
import com.softwareag.entirex.aci.BrokerException;
import com.softwareag.entirex.aci.ConversationState;
import com.softwareag.entirex.aci.UnitofWork;

public class ExxTransportSender extends AbstractHandler implements TransportSender
{
	private ConversationState convState;
	private Broker broker;
	private String brokerID;
	private BrokerService bService;
	
	public ExxTransportSender()
	{
		convState = null;
		broker = null;
		brokerID = null;
		bService = null;
	}
	
	public InvocationResponse invoke(MessageContext msgContext)
			throws AxisFault {
		
		//System.out.println("====== ExxTransportSender.invoke ====== ");

		try{
			
			HashMap hashMap = new HashMap();
			
			if( brokerID == null ){
				EndpointReference epr = msgContext.getTo();
				URL endPointUrl = new URL( epr.getAddress() );
				
				for (String param : endPointUrl.getQuery().split("&")) 
				{
					String pair[] = param.split("=");
					hashMap.put( URLDecoder.decode(pair[0], "UTF-8"), URLDecoder.decode(pair[1], "UTF-8") );
				}

				// 	We get in something like "192.168.135.208:1971:TCP" and we need something like
				// "tcpip://meath"
				brokerID = "tcpip://" + ((String)hashMap.get( "brokerID" )).split(":")[0];
			}

			if( broker == null )
			{
				broker = new Broker(brokerID, "JavaUser");
		
				broker.logon();
			}

			if( bService == null )
			{
				bService = new BrokerService(broker, (String)hashMap.get("class"), (String)hashMap.get("server"), (String)hashMap.get("service") );
			}
		
			UnitofWork uow = new UnitofWork( bService ); 
			
			uow.setDataPersistence( true );

			// set status persistency 5 times longer
			uow.setStatusPersistence( 5 );

			// set lifetime of the Unit of Work to 10 minutes
			uow.setLifetime( "10M" );

			// De-serialize into OutputStream
			OutputStream out = new ByteArrayOutputStream();
			TransportUtils.writeMessage(msgContext, out);
			
			//System.out.println("Sending " + out.toString() );
			BrokerMessage bMessage = new BrokerMessage( out.toString() ); 

			uow.sendCommit( bMessage );
			
			convState = uow.saveState();
			
			//broker.logoff();
		}
		catch( BrokerException be )
		{
			be.printStackTrace();
			throw new AxisFault("Broker Exception");
		}
		catch( Exception e )
		{
			e.printStackTrace();
		}
		
		return InvocationResponse.CONTINUE;
	}

	public void cleanup(MessageContext msgContext) throws AxisFault {
		// TODO Auto-generated method stub
		
	}

	public void init(ConfigurationContext confContext,
			TransportOutDescription transportOut) throws AxisFault {
		// TODO Auto-generated method stub
		throw new AxisFault("TEsTME");
		
	}

	public void stop() {
		// TODO Auto-generated method stub
		
	}
	
	public Object getConvState()
	{
		return (Object)convState;
	}
	
	public void logoff() throws BrokerException
	{
		broker.logoff();
	}
	
	public Object getService()
	{
		return bService;
	}
}
