package com.risaris.axis2.mq.transports;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.net.URL;
import java.net.URLDecoder;
import java.util.HashMap;

import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.TransportOutDescription;
import org.apache.axis2.handlers.AbstractHandler;
import org.apache.axis2.transport.TransportSender;

import com.ibm.mq.*;
import com.ibm.mq.constants.CMQC;

public class MQTransportReceiver extends AbstractHandler implements TransportSender
{
	private byte[] messageID;
	MQEnvironment thisEnv = null;

	public void setMQEnvironment( MQEnvironment env )
	{
		thisEnv = env;
	}
	
	public InvocationResponse invoke(MessageContext msgContext)
			throws AxisFault {
		
		System.out.println("====== MQTransportReceiver.invoke ====== ");

		try{
			
			EndpointReference epr = msgContext.getTo();
			URL endPointUrl = new URL( epr.getAddress() );
			
			HashMap hashMap = new HashMap();
			for (String param : endPointUrl.getQuery().split("&")) 
			{
				String pair[] = param.split("=");
	            hashMap.put( URLDecoder.decode(pair[0], "UTF-8"), URLDecoder.decode(pair[1], "UTF-8") );
			}
			
			if( thisEnv == null )
			{
				MQEnvironment.hostname = (String)hashMap.get("connection");
				MQEnvironment.channel  = (String)hashMap.get("channel");
			}
			//MQEnvironment.port = 1415;
			
			MQQueueManager queueManager=new MQQueueManager( (String)hashMap.get("manager") );

			// Open a Q for input - note open with Shared access
			System.out.println("Using output Q " + (String)hashMap.get("outputQ") );
			
			int openOptions = CMQC.MQOO_INPUT_EXCLUSIVE | CMQC.MQOO_BROWSE;
			
			MQQueue queue = queueManager.accessQueue( (String)hashMap.get("outputQ"), openOptions );
			
			MQMessage myMessage = new MQMessage();
			MQGetMessageOptions gmo = new MQGetMessageOptions();
			gmo.matchOptions = CMQC.MQMO_MATCH_CORREL_ID;
			myMessage.correlationId = messageID;

			// get the message!
			queue.get(myMessage, gmo);  // has default values
			
			// extract message data
			byte[] fullMsg = new byte[ myMessage.getMessageLength() ];
			myMessage.readFully( fullMsg );
           
			//System.out.println("Read : " + new String( fullMsg ) );
			
			InputStream is = new ByteArrayInputStream(fullMsg);
			
			msgContext.setProperty(MessageContext.TRANSPORT_IN, is);
			
			queue.close();
			
			queueManager.disconnect();
			
		}
		catch( MQException mqe )
		{
			if( mqe.completionCode == 2 && mqe.reasonCode == 2033)
			{
				System.err.println("---  No message found on the queue  ---- ");
			}
			else
			{
				mqe.printStackTrace();
			}
		}
		catch( Exception e )
		{
			e.printStackTrace();
		}
		
		return InvocationResponse.CONTINUE;
	}

	public void cleanup(MessageContext msgContext) throws AxisFault {
		// TODO Auto-generated method stub
		
	}

	public void init(ConfigurationContext confContext,
			TransportOutDescription transportOut) throws AxisFault {
		// TODO Auto-generated method stub
		throw new AxisFault("TEsTME");
		
	}

	public void stop() {
		// TODO Auto-generated method stub
		
	}
	
	public void setMessageID(byte[] msgId) {
		 messageID = msgId;
		
	}

}
